// LIBRARIES (Download these!)
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SH110X.h>
#include <Servo.h>

Adafruit_SH1107 display = Adafruit_SH1107(128, 128, &Wire); // OLED Display

// Servo Motors
Servo myServo1;
Servo myServo2;
Servo myServo3;
Servo myServo4;
Servo myServo5;
Servo myServo6;
Servo myServo7;

// Push-Buttons
int button1 = 22;
int button2 = 23;
int button3 = 24;
int button4 = 25;
int button5 = 27;
int button6 = 26;
int button7 = 29;
int button8 = 28;
int button9 = 31;
int button10 = 30;
int button11 = 33;

// Assorted LEDs
int YellowLED = 7;
int GreenLED = 6;
// RGB LED
int redPin = 5;
int greenPin = 4;
int bluePin = 3;
// Piezo Buzzer
int buzzer = 32;

// First Seven-Segment Display (Outputs tens)
int a = A0; int b = 13; int c = A2; int d = A3;
int e = 12; int f = A5; int g = A6;

// Second Seven-Segment Display (Outputs ones)
int a1 = A8; int b1 = 8; int c1 = 11; int d1 = 10;
int e1 = 9; int f1 = A7; int g1 = A4;

// Game variables
int ans = 0;
int digit1 = 0;
int digit2 = 0;
int score = 0;
int firstNum = 0;
int secondNum = 0;
int timeLimit = 10000;  // milliseconds
unsigned long startTime;
int enteredDigits = 0;

void setup() {
  Serial.begin(9600);

  // OLED Display
  display.begin(0x3C);
  display.setRotation(0);
  display.setTextColor(SH110X_WHITE);
  display.setTextSize(1);

  // Assorted LEDs
  pinMode(YellowLED, OUTPUT);
  pinMode(GreenLED, OUTPUT);
  // RGB Led
  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);
  // Buzzer
  pinMode(buzzer, OUTPUT);

  // RGB LED OFF
  setColor(0, 0, 0); 

  // Push-Buttons
  pinMode(button1, INPUT);
  pinMode(button2, INPUT);
  pinMode(button3, INPUT);
  pinMode(button4, INPUT);
  pinMode(button5, INPUT);
  pinMode(button6, INPUT);
  pinMode(button7, INPUT);
  pinMode(button8, INPUT);
  pinMode(button9, INPUT);
  pinMode(button10, INPUT);
  pinMode(button11, INPUT);

  // First Seven-Segment Display (Outputs tens)
  pinMode(a, OUTPUT);
  pinMode(b, OUTPUT);
  pinMode(c, OUTPUT);
  pinMode(d, OUTPUT);
  pinMode(e, OUTPUT);
  pinMode(f, OUTPUT);
  pinMode(g, OUTPUT);

  // Second Seven-Segment Display (Outputs ones)
  pinMode(a1, OUTPUT);
  pinMode(b1, OUTPUT);
  pinMode(c1, OUTPUT);
  pinMode(d1, OUTPUT);
  pinMode(e1, OUTPUT);
  pinMode(f1, OUTPUT);
  pinMode(g1, OUTPUT);

  // Servo Motors
  myServo1.attach(34);
  myServo2.attach(35);
  myServo3.attach(36);
  myServo4.attach(37);
  myServo5.attach(38);
  myServo6.attach(39);
  myServo7.attach(43);

  // All Servo Motors moved down, so they're showing no number during the beginning of the game
  myServo1.write(0);
  myServo2.write(0);
  myServo3.write(0);
  myServo4.write(0);
  myServo5.write(0);
  myServo6.write(0);
  myServo7.write(0);

  // Start Screen
  display.clearDisplay();
  display.setCursor(5, 20);
  display.println("ServoSegmentDisplay");
  display.setCursor(1, 40);
  display.println("A multiplication game");
  display.setCursor(10, 85);
  display.println("Ishandeep Khinda");
  display.setCursor(55, 100);
  display.println(":)");
  display.display();
  delay(5000);

  // Instructions Screen
  display.clearDisplay();
  display.setCursor(0, 5);
  display.println("- You will multiply 2 random numbers given");
  display.setCursor(0, 37);
  display.println("- Then, enter your 1 or 2 digit  answer");
  display.setCursor(0, 68);
  display.println("- Answer are shown   with Servos and      Seven-Segment Display");
  display.setCursor(0, 110);
  display.println("- You have less time  as score increases");
  display.display();
  delay(12000);
  clearDisplays();
}

void loop() {
  // Clear all displays and LEDs before showing home screen
  clearDisplays();
  setColor(0, 0, 0); // Turn off RGB LED
  display.clearDisplay();

  // Display welcome message on OLED
  display.setCursor(0, 10);
  display.println("Press the 'S' button to start playing");
  display.setCursor(0, 50);
  display.println("Get a score of 10 to win the game!");
  display.setCursor(0, 90);
  display.println("HAVE FUN!!!");
  display.setCursor(0, 110);
  display.println(":D");
  display.display();

  Serial.println("Home screen displayed. Waiting for user to start.");

  // Wait until button1 (start button) is pressed and released
  while (digitalRead(button1) == LOW) { delay(10); }
  while (digitalRead(button1) == HIGH) { delay(10); }

  // Start game loop
  while (true) {
    playGame();
  }
}

void playGame() {
  digit1 = 0;
  digit2 = 0;
  enteredDigits = 0;

  // Generate two random numbers (1 to 19) and calculate product
  do {
    firstNum = random(1, 20);
    secondNum = random(1, 20);
    ans = firstNum * secondNum;
  } while (ans >= 99); // Keep generating until product fits 2 digits

  startTime = millis(); // Start timer
  clearDisplays();

  // Loop while time limit not expired
  while (millis() - startTime < timeLimit) {
    // Display question, time remaining, and score on OLED
    display.clearDisplay();
    display.setCursor(0, 10);
    display.print("Question: ");
    display.print(firstNum);
    display.print(" x ");
    display.println(secondNum);

    display.setCursor(0, 30);
    display.print("Time: ");
    display.println((timeLimit - (millis() - startTime)) / 1000);

    display.setCursor(0, 50);
    display.print("Score: ");
    display.println(score);
    display.display();

    // Check if any number button is pressed
    int pressed = checkButtons();

    if (pressed != -1) {
      while (checkButtons() != -1) { delay(10); } // Debounce

      // First digit input
      if (enteredDigits == 0) {
        digit1 = pressed;
        showDigit1(digit1);
        showNumberOnServos(digit1);
        enteredDigits = 1;
        delay(300);
      }
      // Second digit input
      else if (enteredDigits == 1) {
        digit2 = pressed;
        showDigit2(digit2);
        showNumberOnServos(digit2);
        delay(500);

        // Checks if the answer you entered is correct
        int guess = digit1 * 10 + digit2;

        // Check if correct
        if (guess == ans) {
          display.clearDisplay();
          display.setCursor(20, 50);
          display.println("CORRECT");
          display.display();

          setColor(0, 50, 0); // Green light
          tone(buzzer, 1000, 300);
          delay(1000);
          score++;

          // Check for winning condition
          if (score >= 10) {
            display.clearDisplay();
            display.setCursor(10, 50);
            display.println("YOU WIN!");
            display.display();
            setColor(0, 0, 255); // Blue light

            // Shows that you have won with LEDs and Buzzer
            for (int i = 0; i < 5; i++) {
              digitalWrite(GreenLED, HIGH);
              digitalWrite(YellowLED, LOW);
              tone(buzzer, 1000 + i * 100);
              delay(500);
              noTone(buzzer);
              digitalWrite(GreenLED, LOW);
              digitalWrite(YellowLED, HIGH);
              delay(500);
            }

            // Turns everything off, indicating that a new round is starting soon
            setColor(0, 0, 0); // Turn everything off
            digitalWrite(GreenLED, LOW);
            digitalWrite(YellowLED, LOW);
            noTone(buzzer);

            // Countdown before restarting
            for (int i = 5; i >= 1; i--) {
              display.clearDisplay();
              display.setCursor(0, 10);
              display.println("New round starting in");
              display.setCursor(50, 50);
              display.print(i);
              display.println("...");
              display.display();
              delay(1000);
            }

            // Reset game state
            score = 0;
            timeLimit = 10000;
            return;
          }

          // Reduce time limit to increase difficulty
          if (timeLimit > 4000) timeLimit -= 1000;
          return;
        }
        else {
          incorrectReset();
          return;
        }
      }
    }
  }

  // If time ran out without full answer
  incorrectReset();
}

// Check which button is pressed and return corresponding number
int checkButtons() {
  if (digitalRead(button2) == HIGH) return 0;
  if (digitalRead(button3) == HIGH) return 1;
  if (digitalRead(button4) == HIGH) return 2;
  if (digitalRead(button5) == HIGH) return 3;
  if (digitalRead(button6) == HIGH) return 4;
  if (digitalRead(button7) == HIGH) return 5;
  if (digitalRead(button8) == HIGH) return 6;
  if (digitalRead(button9) == HIGH) return 7;
  if (digitalRead(button10) == HIGH) return 8;
  if (digitalRead(button11) == HIGH) return 9;
  return -1;
}

// Handle incorrect answer (wrong guess or timeout)
void incorrectReset() {
  display.clearDisplay();
  display.setCursor(20, 50);
  display.println("INCORRECT");
  display.display();

  setColor(255, 0, 0); // Red light
  tone(buzzer, 200, 1000);
  delay(1500);

  setColor(0, 0, 0);
  digitalWrite(GreenLED, LOW);
  digitalWrite(YellowLED, LOW);
  noTone(buzzer);

  for (int i = 5; i >= 1; i--) {
    display.clearDisplay();
    display.setCursor(0, 10);
    display.println("New round starting in");
    display.setCursor(50, 50);
    display.print(i);
    display.println("...");
    display.display();
    delay(1000);
  }

  score = 0;
  timeLimit = 10000;
  clearDisplays();
}

// Set RGB LED color
void setColor(int red, int green, int blue) {
  red = 255 - red;
  green = 255 - green;
  blue = 255 - blue;
  analogWrite(redPin, red);
  analogWrite(greenPin, green);
  analogWrite(bluePin, blue);
}

// Display number on servos
void showNumberOnServos(int num) {
  digitalWrite(YellowLED, HIGH);
  delay(200);
  if (num == 0) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(75);
    myServo5.write(90);
    myServo6.write(90);
    myServo7.write(0);
  }
  if (num == 1) {
    myServo1.write(0);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(0);
    myServo5.write(0);
    myServo6.write(0);
    myServo7.write(0);
  }
  if (num == 2) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(0);
    myServo4.write(75);
    myServo5.write(90);
    myServo6.write(0);
    myServo7.write(90);
  }
  if (num == 3) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(75);
    myServo5.write(0);
    myServo6.write(0);
    myServo7.write(90);
  }
  if (num == 4) {
    myServo1.write(0);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(0);
    myServo5.write(0);
    myServo6.write(90);
    myServo7.write(90);
  }
  if (num == 5) {
    myServo1.write(90);
    myServo2.write(0);
    myServo3.write(90);
    myServo4.write(75);
    myServo5.write(0);
    myServo6.write(90);
    myServo7.write(90);
  }
  if (num == 6) {
    myServo1.write(90);
    myServo2.write(0);
    myServo3.write(90);
    myServo4.write(75);
    myServo5.write(90);
    myServo6.write(90);
    myServo7.write(90);
  }
  if (num == 7) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(0);
    myServo5.write(0);
    myServo6.write(0);
    myServo7.write(0);
  }
  if (num == 8) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(75);
    myServo5.write(90);
    myServo6.write(90);
    myServo7.write(90);
  }
  if (num == 9) {
    myServo1.write(90);
    myServo2.write(90);
    myServo3.write(90);
    myServo4.write(0);
    myServo5.write(0);
    myServo6.write(90);
    myServo7.write(90);
  }
  digitalWrite(YellowLED, LOW);
  digitalWrite(GreenLED, HIGH);
  delay(500);
  digitalWrite(GreenLED, LOW);
}

// Clear both Seven-Segment Displays
void clearDisplays() {
  digitalWrite(a, HIGH);
  digitalWrite(b, HIGH);
  digitalWrite(c, HIGH);
  digitalWrite(d, HIGH);
  digitalWrite(e, HIGH);
  digitalWrite(f, HIGH);
  digitalWrite(g, HIGH);
  digitalWrite(a1, HIGH);
  digitalWrite(b1, HIGH);
  digitalWrite(c1, HIGH);
  digitalWrite(d1, HIGH);
  digitalWrite(e1, HIGH);
  digitalWrite(f1, HIGH);
  digitalWrite(g1, HIGH);
}

// Show digits on first Seven-Segment display
void showDigit1(int num) {
  if (num == 0) zero();
  if (num == 1) one();
  if (num == 2) two();
  if (num == 3) three();
  if (num == 4) four();
  if (num == 5) five();
  if (num == 6) six();
  if (num == 7) seven();
  if (num == 8) eight();
  if (num == 9) nine();
}

// Show digits on second Seven-Segment display
void showDigit2(int num) {
  if (num == 0) zero1();
  if (num == 1) one1();
  if (num == 2) two1();
  if (num == 3) three1();
  if (num == 4) four1();
  if (num == 5) five1();
  if (num == 6) six1();
  if (num == 7) seven1();
  if (num == 8) eight1();
  if (num == 9) nine1();
}

// FIRST Seven-Segment Display Digits
void zero(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, LOW); 
  digitalWrite(e, LOW); 
  digitalWrite(f, LOW);
  digitalWrite(g, HIGH);
}

void one(){
  digitalWrite(a, HIGH); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, HIGH); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, HIGH);
  digitalWrite(g, HIGH);
}

void two(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, HIGH);
  digitalWrite(d, LOW); 
  digitalWrite(e, LOW); 
  digitalWrite(f, HIGH);
  digitalWrite(g, LOW);
}

void three(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, LOW); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, HIGH);
  digitalWrite(g, LOW);
}

void four(){
  digitalWrite(a, HIGH); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, HIGH); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, LOW);
  digitalWrite(g, LOW);
}

void five(){
  digitalWrite(a, LOW); 
  digitalWrite(b, HIGH); 
  digitalWrite(c, LOW);
  digitalWrite(d, LOW); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, LOW);
  digitalWrite(g, LOW);
}

void six(){
  digitalWrite(a, LOW); 
  digitalWrite(b, HIGH); 
  digitalWrite(c, LOW);
  digitalWrite(d, LOW); 
  digitalWrite(e, LOW);
  digitalWrite(f, LOW);
  digitalWrite(g, LOW);
}

void seven(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, HIGH); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, HIGH);
  digitalWrite(g, HIGH);
}

void eight(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, LOW); 
  digitalWrite(e, LOW); 
  digitalWrite(f, LOW);
  digitalWrite(g, LOW);
}

void nine(){
  digitalWrite(a, LOW); 
  digitalWrite(b, LOW); 
  digitalWrite(c, LOW);
  digitalWrite(d, HIGH); 
  digitalWrite(e, HIGH); 
  digitalWrite(f, LOW);
  digitalWrite(g, LOW);
}

// SECOND` Seven-Segment Display Digits
void zero1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, LOW); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, HIGH);
}

void one1(){
  digitalWrite(a1, HIGH); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, HIGH); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, HIGH);
  digitalWrite(g1, HIGH);
}

void two1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, HIGH);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, LOW); 
  digitalWrite(f1, HIGH);
  digitalWrite(g1, LOW);
}

void three1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, HIGH);
  digitalWrite(g1, LOW);
}

void four1(){
  digitalWrite(a1, HIGH); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, HIGH); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, LOW);
}

void five1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, HIGH); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, LOW);
}

void six1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, HIGH); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, LOW); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, LOW);
}

void seven1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, HIGH); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, HIGH);
  digitalWrite(g1, HIGH);
}

void eight1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, LOW); 
  digitalWrite(e1, LOW); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, LOW);
}

void nine1(){
  digitalWrite(a1, LOW); 
  digitalWrite(b1, LOW); 
  digitalWrite(c1, LOW);
  digitalWrite(d1, HIGH); 
  digitalWrite(e1, HIGH); 
  digitalWrite(f1, LOW);
  digitalWrite(g1, LOW);
}

// THANK YOU :)